@@ -37,7 +37,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtk.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
@@ -131,6 +130,7 @@ send_selection_notify (MsdClipboardManager *manager,
                        Bool                 success)
 {
         XSelectionEvent notify;
+        GdkDisplay     *display;
 
         notify.type = SelectionNotify;
         notify.serial = 0;
@@ -142,7 +142,8 @@ send_selection_notify (MsdClipboardManager *manager,
         notify.property = success ? manager->priv->property : None;
         notify.time = manager->priv->time;
 
-        gdk_error_trap_push ();
+        display = gdk_display_get_default ();
+        gdk_x11_display_error_trap_push (display);
 
         XSendEvent (manager->priv->display,
                     manager->priv->requestor,
@@ -151,7 +152,7 @@ send_selection_notify (MsdClipboardManager *manager,
                     (XEvent *)&notify);
         XSync (manager->priv->display, False);
 
-        gdk_error_trap_pop ();
+        gdk_x11_display_error_trap_pop_ignored (display);
 }
 
 static void
@@ -160,6 +161,7 @@ finish_selection_request (MsdClipboardManager *manager,
                           Bool                 success)
 {
         XSelectionEvent notify;
+        GdkDisplay     *display;
 
         notify.type = SelectionNotify;
         notify.serial = 0;
@@ -171,14 +173,15 @@ finish_selection_request (MsdClipboardManager *manager,
         notify.property = success ? xev->xselectionrequest.property : None;
         notify.time = xev->xselectionrequest.time;
 
-        gdk_error_trap_push ();
+        display = gdk_display_get_default ();
+        gdk_x11_display_error_trap_push (display);
 
         XSendEvent (xev->xselectionrequest.display,
                     xev->xselectionrequest.requestor,
                     False, NoEventMask, (XEvent *) &notify);
         XSync (manager->priv->display, False);
 
-        gdk_error_trap_pop ();
+        gdk_x11_display_error_trap_pop_ignored (display);
 }
 
 static int
@@ -400,12 +403,15 @@ static void
 convert_clipboard_manager (MsdClipboardManager *manager,
                            XEvent              *xev)
 {
+        GdkDisplay    *display;
         Atom          type = None;
         int           format;
         unsigned long nitems;
         unsigned long remaining;
         Atom         *targets = NULL;
 
+        display = gdk_display_get_default ();
+
         if (xev->xselectionrequest.target == XA_SAVE_TARGETS) {
                 if (manager->priv->requestor != None || manager->priv->contents != NULL) {
                         /* We're in the middle of a conversion request, or own
@@ -413,7 +419,7 @@ convert_clipboard_manager (MsdClipboardManager *manager,
                          */
                         finish_selection_request (manager, xev, False);
                 } else {
-                        gdk_error_trap_push ();
+                        gdk_x11_display_error_trap_push (display);
 
                         clipboard_manager_watch_cb (manager,
                                                     xev->xselectionrequest.requestor,
@@ -425,10 +431,10 @@ convert_clipboard_manager (MsdClipboardManager *manager,
                                       StructureNotifyMask);
                         XSync (manager->priv->display, False);
 
-                        if (gdk_error_trap_pop () != Success)
+                        if (gdk_x11_display_error_trap_pop (display) != Success)
                                 return;
 
-                        gdk_error_trap_push ();
+                        gdk_x11_display_error_trap_push (display);
 
                         if (xev->xselectionrequest.property != None) {
                                 XGetWindowProperty (manager->priv->display,
@@ -438,7 +444,7 @@ convert_clipboard_manager (MsdClipboardManager *manager,
                                                     &type, &format, &nitems, &remaining,
                                                     (unsigned char **) &targets);
 
-                                if (gdk_error_trap_pop () != Success) {
+                                if (gdk_x11_display_error_trap_pop (display) != Success) {
                                         if (targets)
                                                 XFree (targets);
 
@@ -488,6 +494,7 @@ static void
 convert_clipboard_target (IncrConversion      *rdata,
                           MsdClipboardManager *manager)
 {
+        GdkDisplay       *display;
         TargetData       *tdata;
         Atom             *targets;
         int               n_targets;
@@ -495,6 +502,8 @@ convert_clipboard_target (IncrConversion      *rdata,
         unsigned long     items;
         XWindowAttributes atts;
 
+        display = gdk_display_get_default ();
+
         if (rdata->target == XA_TARGETS) {
                 n_targets = list_length (manager->priv->contents) + 2;
                 targets = (Atom *) malloc (n_targets * sizeof (Atom));
@@ -541,7 +550,7 @@ convert_clipboard_target (IncrConversion      *rdata,
                         /* start incremental transfer */
                         rdata->offset = 0;
 
-                        gdk_error_trap_push ();
+                        gdk_x11_display_error_trap_push (display);
 
                         XGetWindowAttributes (manager->priv->display, rdata->requestor, &atts);
                         XSelectInput (manager->priv->display, rdata->requestor,
@@ -554,7 +563,7 @@ convert_clipboard_target (IncrConversion      *rdata,
 
                         XSync (manager->priv->display, False);
 
-                        gdk_error_trap_pop ();
+                        gdk_x11_display_error_trap_pop_ignored (display);
                 }
         }
 }
@@ -955,76 +964,11 @@ msd_clipboard_manager_stop (MsdClipboardManager *manager)
         list_free (manager->priv->contents);
 }
 
-static void
-msd_clipboard_manager_set_property (GObject        *object,
-                                    guint           prop_id,
-                                    const GValue   *value,
-                                    GParamSpec     *pspec)
-{
-        MsdClipboardManager *self;
-
-        self = MSD_CLIPBOARD_MANAGER (object);
-
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static void
-msd_clipboard_manager_get_property (GObject        *object,
-                                    guint           prop_id,
-                                    GValue         *value,
-                                    GParamSpec     *pspec)
-{
-        MsdClipboardManager *self;
-
-        self = MSD_CLIPBOARD_MANAGER (object);
-
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static GObject *
-msd_clipboard_manager_constructor (GType                  type,
-                                   guint                  n_construct_properties,
-                                   GObjectConstructParam *construct_properties)
-{
-        MsdClipboardManager      *clipboard_manager;
-        MsdClipboardManagerClass *klass;
-
-        klass = MSD_CLIPBOARD_MANAGER_CLASS (g_type_class_peek (MSD_TYPE_CLIPBOARD_MANAGER));
-
-        clipboard_manager = MSD_CLIPBOARD_MANAGER (G_OBJECT_CLASS (msd_clipboard_manager_parent_class)->constructor (type,
-                                                                                                      n_construct_properties,
-                                                                                                      construct_properties));
-
-        return G_OBJECT (clipboard_manager);
-}
-
-static void
-msd_clipboard_manager_dispose (GObject *object)
-{
-        MsdClipboardManager *clipboard_manager;
-
-        clipboard_manager = MSD_CLIPBOARD_MANAGER (object);
-
-        G_OBJECT_CLASS (msd_clipboard_manager_parent_class)->dispose (object);
-}
-
 static void
 msd_clipboard_manager_class_init (MsdClipboardManagerClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->get_property = msd_clipboard_manager_get_property;
-        object_class->set_property = msd_clipboard_manager_set_property;
-        object_class->constructor = msd_clipboard_manager_constructor;
-        object_class->dispose = msd_clipboard_manager_dispose;
         object_class->finalize = msd_clipboard_manager_finalize;
 
         g_type_class_add_private (klass, sizeof (MsdClipboardManagerPrivate));
