@@ -73,7 +73,9 @@ struct _MsdMediaKeysManagerPrivate
         /* Volume bits */
         MateMixerContext       *context;
         MateMixerStream        *stream;
+        MateMixerStream        *source_stream;
         MateMixerStreamControl *control;
+        MateMixerStreamControl *source_control;
 #endif
         GtkWidget        *dialog;
         GSettings        *settings;
@@ -83,6 +85,11 @@ struct _MsdMediaKeysManagerPrivate
         GdkScreen        *current_screen;
         GSList           *screens;
 
+        /* RFKill stuff */
+        guint            rfkill_watch_id;
+        GDBusProxy      *rfkill_proxy;
+        GCancellable    *rfkill_cancellable;
+
         GList            *media_players;
 
         DBusGConnection  *connection;
@@ -388,6 +395,18 @@ static void init_kbd(MsdMediaKeysManager* manager)
 	mate_settings_profile_end(NULL);
 }
 
+static void
+ensure_cancellable (GCancellable **cancellable)
+{
+        if (*cancellable == NULL) {
+                *cancellable = g_cancellable_new ();
+                g_object_add_weak_pointer (G_OBJECT (*cancellable),
+                                           (gpointer *)cancellable);
+        } else {
+                g_object_ref (*cancellable);
+        }
+}
+
 static void
 dialog_show (MsdMediaKeysManager *manager)
 {
@@ -397,9 +416,9 @@ dialog_show (MsdMediaKeysManager *manager)
         int            screen_h;
         int            x;
         int            y;
-        GdkDisplay *display;
-        GdkDeviceManager *device_manager;
-        GdkDevice *pointer;
+        GdkDisplay    *display;
+        GdkSeat       *seat;
+        GdkDevice     *pointer;
         int            pointer_x;
         int            pointer_y;
         GtkRequisition win_req;
@@ -431,8 +450,8 @@ dialog_show (MsdMediaKeysManager *manager)
 
         pointer_screen = NULL;
         display = gdk_screen_get_display (manager->priv->current_screen);
-        device_manager = gdk_display_get_device_manager (display);
-        pointer = gdk_device_manager_get_client_pointer (device_manager);
+        seat = gdk_display_get_default_seat (display);
+        pointer = gdk_seat_get_pointer (seat);
 
         gdk_device_get_position (pointer,
                                  &pointer_screen,
@@ -507,6 +526,22 @@ do_media_action (MsdMediaKeysManager *manager)
         }
 }
 
+static void
+do_calculator_action (MsdMediaKeysManager *manager)
+{
+        GSettings *settings;
+        char *calc;
+
+        settings = g_settings_new ("org.mate.applications-calculator");
+        calc = g_settings_get_string (settings, "exec");
+
+        if (calc)
+                execute (manager, calc, FALSE, FALSE);
+
+        g_free (calc);
+        g_object_unref (settings);
+}
+
 static void
 do_shutdown_action (MsdMediaKeysManager *manager)
 {
@@ -565,7 +600,7 @@ do_eject_action (MsdMediaKeysManager *manager)
         dialog_init (manager);
         msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
                                                  "media-eject",
-                                                 FALSE);
+                                                 NULL);
         dialog_show (manager);
 
         /* Clean up the drive selection and exit if no suitable
@@ -585,6 +620,16 @@ do_eject_action (MsdMediaKeysManager *manager)
         g_object_unref (fav_drive);
 }
 
+static void
+do_touchpad_osd_action (MsdMediaKeysManager *manager, gboolean state)
+{
+        dialog_init (manager);
+        msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                 state ? "input-touchpad" : "touchpad-disabled",
+                                                 state ? _("Touchpad enabled") : _("Touchpad disabled"));
+        dialog_show (manager);
+}
+
 static void
 do_touchpad_action (MsdMediaKeysManager *manager)
 {
@@ -592,17 +637,11 @@ do_touchpad_action (MsdMediaKeysManager *manager)
         gboolean state = g_settings_get_boolean (settings, TOUCHPAD_ENABLED_KEY);
 
         if (touchpad_is_present () == FALSE) {
-                dialog_init (manager);
-                msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
-                                                         "touchpad-disabled", FALSE);
+                do_touchpad_osd_action (manager, FALSE);
                 return;
         }
 
-        dialog_init (manager);
-        msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
-                                                 (!state) ? "touchpad-enabled" : "touchpad-disabled",
-                                                 FALSE);
-        dialog_show (manager);
+        do_touchpad_osd_action (manager, !state);
 
         g_settings_set_boolean (settings, TOUCHPAD_ENABLED_KEY, !state);
         g_object_unref (settings);
@@ -613,12 +652,22 @@ static void
 update_dialog (MsdMediaKeysManager *manager,
                guint                volume,
                gboolean             muted,
-               gboolean             sound_changed)
+               gboolean             sound_changed,
+               gboolean             quiet,
+               gboolean             is_mic)
 {
+        if (muted)
+                volume = 0.0;
+
         dialog_init (manager);
 
-        msd_media_keys_window_set_volume_muted (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
-                                                muted);
+        if (is_mic)
+                msd_media_keys_window_set_mic_muted (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                     muted);
+        else
+                msd_media_keys_window_set_volume_muted (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                        muted);
+
         msd_media_keys_window_set_volume_level (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
                                                 volume);
 
@@ -627,7 +676,7 @@ update_dialog (MsdMediaKeysManager *manager,
         dialog_show (manager);
 
 #ifdef HAVE_LIBCANBERRA
-        if (sound_changed != FALSE && muted == FALSE)
+        if (quiet == FALSE && sound_changed != FALSE && muted == FALSE && is_mic == FALSE)
                 ca_gtk_play_for_widget (manager->priv->dialog, 0,
                                         CA_PROP_EVENT_ID, "audio-volume-change",
                                         CA_PROP_EVENT_DESCRIPTION, "Volume changed through key press",
@@ -639,7 +688,9 @@ update_dialog (MsdMediaKeysManager *manager,
 }
 
 static void
-do_sound_action (MsdMediaKeysManager *manager, int type)
+do_sound_action (MsdMediaKeysManager *manager,
+                 int type,
+                 gboolean quiet)
 {
         gboolean muted;
         gboolean muted_last;
@@ -648,14 +699,21 @@ do_sound_action (MsdMediaKeysManager *manager, int type)
         guint    volume_min, volume_max;
         guint    volume_step;
         guint    volume_last;
+        MateMixerStreamControl *control;
 
-        if (manager->priv->control == NULL)
+        gboolean is_input_control =
+                type == MIC_MUTE_KEY ? TRUE : FALSE;
+        if (is_input_control)
+                control = manager->priv->source_control;
+        else
+                control = manager->priv->control;
+        if (control == NULL)
                 return;
 
         /* Theoretically the volume limits might be different for different
          * streams, also the minimum might not always start at 0 */
-        volume_min = mate_mixer_stream_control_get_min_volume (manager->priv->control);
-        volume_max = mate_mixer_stream_control_get_normal_volume (manager->priv->control);
+        volume_min = mate_mixer_stream_control_get_min_volume (control);
+        volume_max = mate_mixer_stream_control_get_normal_volume (control);
 
         volume_step = g_settings_get_int (manager->priv->settings, "volume-step");
         if (volume_step <= 0 ||
@@ -666,12 +724,13 @@ do_sound_action (MsdMediaKeysManager *manager, int type)
         volume_step = (volume_max - volume_min) * volume_step / 100;
 
         volume = volume_last =
-                mate_mixer_stream_control_get_volume (manager->priv->control);
+                mate_mixer_stream_control_get_volume (control);
         muted = muted_last =
-                mate_mixer_stream_control_get_mute (manager->priv->control);
+                mate_mixer_stream_control_get_mute (control);
 
         switch (type) {
         case MUTE_KEY:
+        case MIC_MUTE_KEY:
                 muted = !muted;
                 break;
         case VOLUME_DOWN_KEY:
@@ -696,13 +755,14 @@ do_sound_action (MsdMediaKeysManager *manager, int type)
         }
 
         if (muted != muted_last) {
-                if (mate_mixer_stream_control_set_mute (manager->priv->control, muted))
+                if (mate_mixer_stream_control_set_mute (control, muted))
                         sound_changed = TRUE;
                 else
                         muted = muted_last;
         }
-        if (volume != mate_mixer_stream_control_get_volume (manager->priv->control)) {
-                if (mate_mixer_stream_control_set_volume (manager->priv->control, volume))
+
+        if (volume != mate_mixer_stream_control_get_volume (control)) {
+                if (mate_mixer_stream_control_set_volume (control, volume))
                         sound_changed = TRUE;
                 else
                         volume = volume_last;
@@ -711,7 +771,9 @@ do_sound_action (MsdMediaKeysManager *manager, int type)
         update_dialog (manager,
                        CLAMP (100 * volume / (volume_max - volume_min), 0, 100),
                        muted,
-                       sound_changed);
+                       sound_changed,
+                       quiet,
+                       is_input_control);
 }
 
 static void
@@ -747,12 +809,45 @@ update_default_output (MsdMediaKeysManager *manager)
                 g_debug ("Default output stream unset");
 }
 
+static void
+update_default_input (MsdMediaKeysManager *manager)
+{
+        MateMixerStream        *stream;
+        MateMixerStreamControl *control = NULL;
+
+        stream = mate_mixer_context_get_default_input_stream (manager->priv->context);
+        if (stream != NULL)
+                control = mate_mixer_stream_get_default_control (stream);
+
+        if (stream == manager->priv->source_stream)
+                return;
+
+        g_clear_object (&manager->priv->source_stream);
+        g_clear_object (&manager->priv->source_control);
+
+        if (control != NULL) {
+                MateMixerStreamControlFlags flags = mate_mixer_stream_control_get_flags (control);
+
+                /* Do not use the stream if it is not possible to mute it or
+                 * change the volume */
+                if (!(flags & MATE_MIXER_STREAM_CONTROL_MUTE_WRITABLE))
+                        return;
+
+                manager->priv->source_stream  = g_object_ref (stream);
+                manager->priv->source_control = g_object_ref (control);
+                g_debug ("Default input stream updated to %s",
+                         mate_mixer_stream_get_name (stream));
+        } else
+                g_debug ("Default input stream unset");
+}
+
 static void
 on_context_state_notify (MateMixerContext    *context,
                          GParamSpec          *pspec,
                          MsdMediaKeysManager *manager)
 {
         update_default_output (manager);
+        update_default_input (manager);
 }
 
 static void
@@ -763,6 +858,14 @@ on_context_default_output_notify (MateMixerContext    *context,
         update_default_output (manager);
 }
 
+static void
+on_context_default_input_notify (MateMixerContext    *context,
+                                 GParamSpec          *pspec,
+                                 MsdMediaKeysManager *manager)
+{
+        update_default_input (manager);
+}
+
 static void
 on_context_stream_removed (MateMixerContext    *context,
                            const gchar         *name,
@@ -777,9 +880,153 @@ on_context_stream_removed (MateMixerContext    *context,
                         g_clear_object (&manager->priv->control);
                 }
         }
+        if (manager->priv->source_stream != NULL) {
+                MateMixerStream *stream =
+                        mate_mixer_context_get_stream (manager->priv->context, name);
+
+                if (stream == manager->priv->source_stream) {
+                        g_clear_object (&manager->priv->source_stream);
+                        g_clear_object (&manager->priv->source_control);
+                }
+        }
 }
 #endif /* HAVE_LIBMATEMIXER */
 
+static gboolean
+get_rfkill_property (MsdMediaKeysManager *manager,
+                     const char          *property)
+{
+        GVariant *v;
+        gboolean ret;
+
+        v = g_dbus_proxy_get_cached_property (manager->priv->rfkill_proxy, property);
+        if (!v)
+                return FALSE;
+        ret = g_variant_get_boolean (v);
+        g_variant_unref (v);
+
+        return ret;
+}
+
+typedef struct {
+        MsdMediaKeysManager *manager;
+        char *property;
+        gboolean bluetooth;
+        gboolean target_state;
+} RfkillData;
+
+static void
+set_rfkill_complete (GObject      *object,
+                     GAsyncResult *result,
+                     gpointer      user_data)
+{
+        GError *error = NULL;
+        GVariant *variant;
+        RfkillData *data = user_data;
+
+        variant = g_dbus_proxy_call_finish (G_DBUS_PROXY (object), result, &error);
+
+        if (variant == NULL) {
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Failed to set '%s' property: %s", data->property, error->message);
+                g_error_free (error);
+                goto out;
+        }
+        g_variant_unref (variant);
+
+        g_debug ("Finished changing rfkill, property %s is now %s",
+                 data->property, data->target_state ? "true" : "false");
+
+        if (data->bluetooth){
+                if (data->target_state)
+                        msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (data->manager->priv->dialog),
+                                                                 "bluetooth-disabled-symbolic", _("Bluetooth disabled"));
+                else
+                        msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (data->manager->priv->dialog),
+                                                                 "bluetooth-active-symbolic", _("Bluetooth enabled"));
+        } else {
+                if (data->target_state)
+                        msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (data->manager->priv->dialog),
+                                                                 "airplane-mode-symbolic", _("Airplane mode enabled"));
+                else
+                        msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (data->manager->priv->dialog),
+                                                                 "network-wireless-signal-excellent-symbolic", _("Airplane mode disabled"));
+        }
+        dialog_show (data->manager);
+out:
+        g_free (data->property);
+        g_free (data);
+}
+
+static void
+do_rfkill_action (MsdMediaKeysManager *manager,
+                  gboolean             bluetooth)
+{
+        const char *has_mode, *hw_mode, *mode;
+        gboolean new_state;
+        RfkillData *data;
+
+        dialog_init (manager);
+
+        has_mode = bluetooth ? "BluetoothHasAirplaneMode" : "HasAirplaneMode";
+        hw_mode = bluetooth ? "BluetoothHardwareAirplaneMode" : "HardwareAirplaneMode";
+        mode = bluetooth ? "BluetoothAirplaneMode" : "AirplaneMode";
+
+        if (manager->priv->rfkill_proxy == NULL)
+                return;
+
+        if (get_rfkill_property (manager, has_mode) == FALSE)
+                return;
+
+        if (get_rfkill_property (manager, hw_mode)) {
+                msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                        "airplane-mode-symbolic",
+                                                        _("Hardware Airplane Mode"));
+                dialog_show (manager);
+                return;
+        }
+
+        new_state = !get_rfkill_property (manager, mode);
+        data = g_new0 (RfkillData, 1);
+        data->manager = manager;
+        data->property = g_strdup (mode);
+        data->bluetooth = bluetooth;
+        data->target_state = new_state;
+        g_dbus_proxy_call (manager->priv->rfkill_proxy,
+                           "org.freedesktop.DBus.Properties.Set",
+                           g_variant_new ("(ssv)",
+                                          "org.mate.SettingsDaemon.Rfkill",
+                                          data->property,
+                                          g_variant_new_boolean (new_state)),
+                           G_DBUS_CALL_FLAGS_NONE, -1,
+                           manager->priv->rfkill_cancellable,
+                           set_rfkill_complete, data);
+
+        g_debug ("Setting rfkill property %s to %s",
+                 data->property, new_state ? "true" : "false");
+}
+
+static void
+do_display_osd_action (MsdMediaKeysManager *manager)
+{
+        GdkDisplay *display;
+        int         n_monitors;
+
+        display = gdk_display_get_default ();
+        n_monitors = gdk_display_get_n_monitors (display);
+
+        dialog_init (manager);
+        if (n_monitors > 1)
+                msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                         "video-joined-displays-symbolic",
+                                                         _("Changing Screen Layout"));
+        else
+                msd_media_keys_window_set_action_custom (MSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
+                                                         "video-single-display-symbolic",
+                                                         _("No External Display"));
+        dialog_show (manager);
+}
+
 static gint
 find_by_application (gconstpointer a,
                      gconstpointer b)
@@ -930,11 +1177,33 @@ do_action (MsdMediaKeysManager *manager,
         case TOUCHPAD_KEY:
                 do_touchpad_action (manager);
                 break;
+        case TOUCHPAD_ON_KEY:
+                do_touchpad_osd_action(manager, TRUE);
+                break;
+        case TOUCHPAD_OFF_KEY:
+                do_touchpad_osd_action(manager, FALSE);
+                break;
         case MUTE_KEY:
         case VOLUME_DOWN_KEY:
         case VOLUME_UP_KEY:
+        case MIC_MUTE_KEY:
 #ifdef HAVE_LIBMATEMIXER
-                do_sound_action (manager, type);
+                do_sound_action (manager, type, FALSE);
+#endif
+                break;
+        case MUTE_QUIET_KEY:
+#ifdef HAVE_LIBMATEMIXER
+                do_sound_action (manager, MUTE_KEY, TRUE);
+#endif
+                break;
+        case VOLUME_DOWN_QUIET_KEY:
+#ifdef HAVE_LIBMATEMIXER
+                do_sound_action (manager, VOLUME_DOWN_KEY, TRUE);
+#endif
+                break;
+        case VOLUME_UP_QUIET_KEY:
+#ifdef HAVE_LIBMATEMIXER
+                do_sound_action (manager, VOLUME_UP_KEY, TRUE);
 #endif
                 break;
         case POWER_KEY:
@@ -967,6 +1236,12 @@ do_action (MsdMediaKeysManager *manager,
         case EMAIL_KEY:
                 do_url_action (manager, "mailto");
                 break;
+        case CONTROL_CENTER_KEY:
+                if ((cmd = g_find_program_in_path ("mate-control-center")))
+                        execute (manager, "mate-control-center", FALSE, FALSE);
+
+                g_free (cmd);
+                break;
         case SCREENSAVER_KEY:
                 if ((cmd = g_find_program_in_path ("mate-screensaver-command"))) {
                         execute (manager, "mate-screensaver-command --lock", FALSE, FALSE);
@@ -986,15 +1261,7 @@ do_action (MsdMediaKeysManager *manager,
                 do_media_action (manager);
                 break;
         case CALCULATOR_KEY:
-                if ((cmd = g_find_program_in_path ("galculator"))) {
-                        execute (manager, "galculator", FALSE, FALSE);
-                } else if ((cmd = g_find_program_in_path ("mate-calc"))) {
-                        execute (manager, "mate-calc", FALSE, FALSE);
-                } else {
-                        execute (manager, "gnome-calculator", FALSE, FALSE);
-                }
-
-                g_free (cmd);
+                do_calculator_action (manager);
                 break;
         case PLAY_KEY:
                 return do_multimedia_player_action (manager, "Play");
@@ -1023,6 +1290,15 @@ do_action (MsdMediaKeysManager *manager,
         case ON_SCREEN_KEYBOARD_KEY:
                 do_on_screen_keyboard_action (manager);
                 break;
+        case RFKILL_KEY:
+                do_rfkill_action (manager, FALSE);
+                break;
+        case BLUETOOTH_RFKILL_KEY:
+                do_rfkill_action (manager, TRUE);
+                break;
+        case DISPLAY_KEY:
+                do_display_osd_action (manager);
+                break;
         default:
                 g_assert_not_reached ();
         }
@@ -1070,6 +1346,8 @@ acme_filter_events (GdkXEvent           *xevent,
                         switch (keys[i].key_type) {
                         case VOLUME_DOWN_KEY:
                         case VOLUME_UP_KEY:
+                        case VOLUME_DOWN_QUIET_KEY:
+                        case VOLUME_UP_QUIET_KEY:
                                 /* auto-repeatable keys */
                                 if (xev->type != KeyPress) {
                                         return GDK_FILTER_CONTINUE;
@@ -1090,6 +1368,34 @@ acme_filter_events (GdkXEvent           *xevent,
         return GDK_FILTER_CONTINUE;
 }
 
+static void
+on_rfkill_proxy_ready (GObject      *source,
+                       GAsyncResult *result,
+                       gpointer      data)
+{
+        MsdMediaKeysManager *manager = data;
+
+        manager->priv->rfkill_proxy =
+                g_dbus_proxy_new_for_bus_finish (result, NULL);
+}
+
+static void
+rfkill_appeared_cb (GDBusConnection *connection,
+                    const gchar     *name,
+                    const gchar     *name_owner,
+                    gpointer         user_data)
+{
+        MsdMediaKeysManager *manager = user_data;
+
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                                  0, NULL,
+                                  "org.mate.SettingsDaemon.Rfkill",
+                                  "/org/mate/SettingsDaemon/Rfkill",
+                                  "org.mate.SettingsDaemon.Rfkill",
+                                  manager->priv->rfkill_cancellable,
+                                  on_rfkill_proxy_ready, manager);
+}
+
 static gboolean
 start_media_keys_idle_cb (MsdMediaKeysManager *manager)
 {
@@ -1106,6 +1412,8 @@ start_media_keys_idle_cb (MsdMediaKeysManager *manager)
         manager->priv->volume_monitor = g_volume_monitor_get ();
         manager->priv->settings = g_settings_new (BINDING_SCHEMA);
 
+        ensure_cancellable (&manager->priv->rfkill_cancellable);
+
         init_screens (manager);
         init_kbd (manager);
 
@@ -1136,6 +1444,13 @@ start_media_keys_idle_cb (MsdMediaKeysManager *manager)
                 mate_settings_profile_end ("gdk_window_add_filter");
         }
 
+        manager->priv->rfkill_watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+                                                           "org.mate.SettingsDaemon.Rfkill",
+                                                           G_BUS_NAME_WATCHER_FLAGS_NONE,
+                                                           rfkill_appeared_cb,
+                                                           NULL,
+                                                           manager, NULL);
+
         mate_settings_profile_end (NULL);
 
         return FALSE;
@@ -1160,6 +1475,10 @@ msd_media_keys_manager_start (MsdMediaKeysManager *manager, GError **error)
                                   "notify::default-output-stream",
                                   G_CALLBACK (on_context_default_output_notify),
                                   manager);
+                g_signal_connect (manager->priv->context,
+                                  "notify::default-input-stream",
+                                  G_CALLBACK (on_context_default_input_notify),
+                                  manager);
                 g_signal_connect (manager->priv->context,
                                   "stream-removed",
                                   G_CALLBACK (on_context_stream_removed),
@@ -1195,6 +1514,11 @@ msd_media_keys_manager_stop (MsdMediaKeysManager *manager)
                                           manager);
         }
 
+        if (manager->priv->rfkill_watch_id > 0) {
+                g_bus_unwatch_name (manager->priv->rfkill_watch_id);
+                manager->priv->rfkill_watch_id = 0;
+        }
+
         if (priv->settings != NULL) {
                 g_object_unref (priv->settings);
                 priv->settings = NULL;
@@ -1233,9 +1557,16 @@ msd_media_keys_manager_stop (MsdMediaKeysManager *manager)
         g_slist_free (priv->screens);
         priv->screens = NULL;
 
+        if (priv->rfkill_cancellable != NULL) {
+                g_cancellable_cancel (priv->rfkill_cancellable);
+                g_clear_object (&priv->rfkill_cancellable);
+        }
+
 #ifdef HAVE_LIBMATEMIXER
         g_clear_object (&priv->stream);
+        g_clear_object (&priv->source_stream);
         g_clear_object (&priv->control);
+        g_clear_object (&priv->source_control);
         g_clear_object (&priv->context);
 #endif
 
