@@ -34,17 +34,22 @@
 
 #define MSD_MEDIA_KEYS_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MSD_TYPE_MEDIA_KEYS_WINDOW, MsdMediaKeysWindowPrivate))
 
+#define ICON_SCALE 0.55           /* size of the icon compared to the whole OSD */
+
 struct MsdMediaKeysWindowPrivate
 {
         MsdMediaKeysWindowAction action;
         char                    *icon_name;
-        gboolean                 show_level;
+        char                    *description;
 
         guint                    volume_muted : 1;
+        guint                    mic_muted : 1;
+        guint                    is_mic :1;
         int                      volume_level;
 
         GtkImage                *image;
         GtkWidget               *progress;
+        GtkWidget               *label;
 };
 
 G_DEFINE_TYPE (MsdMediaKeysWindow, msd_media_keys_window, MSD_TYPE_OSD_WINDOW)
@@ -63,6 +68,18 @@ volume_controls_set_visible (MsdMediaKeysWindow *window,
         }
 }
 
+static void
+description_label_set_visible (MsdMediaKeysWindow *window,
+                               gboolean            visible)
+{
+        if (visible) {
+                gtk_label_set_text (GTK_LABEL (window->priv->label), window->priv->description);
+                gtk_widget_show (window->priv->label);
+        } else {
+                gtk_widget_hide (window->priv->label);
+        }
+}
+
 static void
 window_set_icon_name (MsdMediaKeysWindow *window,
                       const char         *name)
@@ -81,16 +98,24 @@ action_changed (MsdMediaKeysWindow *window)
                 switch (window->priv->action) {
                 case MSD_MEDIA_KEYS_WINDOW_ACTION_VOLUME:
                         volume_controls_set_visible (window, TRUE);
+                        description_label_set_visible (window, FALSE);
 
-                        if (window->priv->volume_muted) {
-                                window_set_icon_name (window, "audio-volume-muted");
+                        if (window->priv->is_mic) {
+                                if (window->priv->mic_muted)
+                                        window_set_icon_name (window, "microphone-sensitivity-muted");
+                                else
+                                        window_set_icon_name (window, "microphone-sensitivity-high");
                         } else {
-                                window_set_icon_name (window, "audio-volume-high");
+                                if (window->priv->volume_muted)
+                                        window_set_icon_name (window, "audio-volume-muted");
+                                else
+                                        window_set_icon_name (window, "audio-volume-high");
                         }
 
                         break;
                 case MSD_MEDIA_KEYS_WINDOW_ACTION_CUSTOM:
-                        volume_controls_set_visible (window, window->priv->show_level);
+                        volume_controls_set_visible (window, FALSE);
+                        description_label_set_visible (window, TRUE);
                         window_set_icon_name (window, window->priv->icon_name);
                         break;
                 default:
@@ -131,6 +156,20 @@ volume_muted_changed (MsdMediaKeysWindow *window)
         }
 }
 
+static void
+mic_muted_changed (MsdMediaKeysWindow *window)
+{
+        msd_osd_window_update_and_hide (MSD_OSD_WINDOW (window));
+
+        if (!msd_osd_window_is_composited (MSD_OSD_WINDOW (window))) {
+                if (window->priv->mic_muted) {
+                        window_set_icon_name (window, "microphone-sensitivity-muted");
+                } else {
+                        window_set_icon_name (window, "microphone-sensitivity-high");
+                }
+        }
+}
+
 void
 msd_media_keys_window_set_action (MsdMediaKeysWindow      *window,
                                   MsdMediaKeysWindowAction action)
@@ -142,6 +181,17 @@ msd_media_keys_window_set_action (MsdMediaKeysWindow      *window,
                 window->priv->action = action;
                 action_changed (window);
         } else {
+                if (window->priv->is_mic) {
+                        if (window->priv->mic_muted)
+                                window_set_icon_name (window, "microphone-sensitivity-muted");
+                        else
+                                window_set_icon_name (window, "microphone-sensitivity-high");
+                } else {
+                        if (window->priv->volume_muted)
+                                window_set_icon_name (window, "audio-volume-muted");
+                        else
+                                window_set_icon_name (window, "audio-volume-high");
+                }
                 msd_osd_window_update_and_hide (MSD_OSD_WINDOW (window));
         }
 }
@@ -149,18 +199,19 @@ msd_media_keys_window_set_action (MsdMediaKeysWindow      *window,
 void
 msd_media_keys_window_set_action_custom (MsdMediaKeysWindow      *window,
                                          const char              *icon_name,
-                                         gboolean                 show_level)
+                                         const char              *description)
 {
         g_return_if_fail (MSD_IS_MEDIA_KEYS_WINDOW (window));
         g_return_if_fail (icon_name != NULL);
 
         if (window->priv->action != MSD_MEDIA_KEYS_WINDOW_ACTION_CUSTOM ||
             g_strcmp0 (window->priv->icon_name, icon_name) != 0 ||
-            window->priv->show_level != show_level) {
+            g_strcmp0 (window->priv->description, description) != 0) {
                 window->priv->action = MSD_MEDIA_KEYS_WINDOW_ACTION_CUSTOM;
                 g_free (window->priv->icon_name);
                 window->priv->icon_name = g_strdup (icon_name);
-                window->priv->show_level = show_level;
+                g_free (window->priv->description);
+                window->priv->description = g_strdup (description);
                 action_changed (window);
         } else {
                 msd_osd_window_update_and_hide (MSD_OSD_WINDOW (window));
@@ -177,6 +228,20 @@ msd_media_keys_window_set_volume_muted (MsdMediaKeysWindow *window,
                 window->priv->volume_muted = muted;
                 volume_muted_changed (window);
         }
+        window->priv->is_mic = FALSE;
+}
+
+void
+msd_media_keys_window_set_mic_muted (MsdMediaKeysWindow *window,
+                                     gboolean            muted)
+{
+        g_return_if_fail (MSD_IS_MEDIA_KEYS_WINDOW (window));
+
+        if (window->priv->mic_muted != muted) {
+                window->priv->mic_muted = muted;
+                mic_muted_changed (window);
+        }
+        window->priv->is_mic = TRUE;
 }
 
 void
@@ -361,18 +426,35 @@ render_speaker (MsdMediaKeysWindow *window,
                 "audio-volume-low",
                 "audio-volume-medium",
                 "audio-volume-high",
+                "microphone-sensitivity-muted",
+                "microphone-sensitivity-low",
+                "microphone-sensitivity-medium",
+                "microphone-sensitivity-high",
                 NULL
         };
-
-        if (window->priv->volume_muted) {
-                n = 0;
+        if (!window->priv->is_mic) {
+                if (window->priv->volume_muted) {
+                        n = 0;
+                } else {
+                        /* select volume image */
+                        n = 3 * window->priv->volume_level / 100 + 1;
+                        if (n < 1) {
+                                n = 1;
+                        } else if (n > 3) {
+                                n = 3;
+                        }
+                }
         } else {
-                /* select image */
-                n = 3 * window->priv->volume_level / 100 + 1;
-                if (n < 1) {
-                        n = 1;
-                } else if (n > 3) {
-                        n = 3;
+                if (window->priv->mic_muted) {
+                        n = 4;
+                } else {
+                        /* select microphone image */
+                        n = 3 * window->priv->volume_level / 100 + 5;
+                        if (n < 5) {
+                                n = 5;
+                        } else if (n > 7) {
+                                n = 7;
+                        }
                 }
         }
 
@@ -392,6 +474,23 @@ render_speaker (MsdMediaKeysWindow *window,
         return TRUE;
 }
 
+static void
+draw_description_label (MsdMediaKeysWindow *window,
+                        cairo_t            *cr,
+                        double              _y0,
+                        double              width)
+{
+        cairo_text_extents_t extents;
+
+        cairo_set_source_rgb (cr, 1, 1, 1);
+        cairo_set_font_size (cr, 14);
+
+        /* centered text */
+        cairo_text_extents (cr, window->priv->description, &extents);
+        cairo_move_to (cr, width / 2 - extents.width / 2, _y0);
+        cairo_show_text (cr, window->priv->description);
+}
+
 static void
 draw_volume_boxes (MsdMediaKeysWindow *window,
                    cairo_t            *cr,
@@ -449,15 +548,15 @@ draw_action_volume (MsdMediaKeysWindow *window,
 
         gtk_window_get_size (GTK_WINDOW (window), &window_width, &window_height);
 
-        icon_box_width = round (window_width * 0.65);
-        icon_box_height = round (window_height * 0.65);
+        icon_box_width = round (window_width * ICON_SCALE);
+        icon_box_height = round (window_height * ICON_SCALE);
         volume_box_width = icon_box_width;
         volume_box_height = round (window_height * 0.05);
 
-        icon_box_x0 = (window_width - icon_box_width) / 2;
-        icon_box_y0 = (window_height - icon_box_height - volume_box_height) / 2;
+        icon_box_x0 = round ((window_width - icon_box_width) / 2);
+        icon_box_y0 = round ((window_height - icon_box_height) / 2);
         volume_box_x0 = round (icon_box_x0);
-        volume_box_y0 = round (icon_box_height + icon_box_y0);
+        volume_box_y0 = round (window_height - icon_box_y0 / 2 - volume_box_height);
 
 #if 0
         g_message ("icon box: w=%f h=%f _x0=%f _y0=%f",
@@ -578,23 +677,21 @@ draw_action_custom (MsdMediaKeysWindow *window,
         double icon_box_height;
         double icon_box_x0;
         double icon_box_y0;
-        double bright_box_x0;
-        double bright_box_y0;
-        double bright_box_width;
-        double bright_box_height;
+        double label_box_y0;
+        double label_box_width;
+        double label_box_height;
         gboolean res;
 
         gtk_window_get_size (GTK_WINDOW (window), &window_width, &window_height);
 
-        icon_box_width = round (window_width * 0.65);
-        icon_box_height = round (window_height * 0.65);
-        bright_box_width = round (icon_box_width);
-        bright_box_height = round (window_height * 0.05);
+        icon_box_width = round (window_width * ICON_SCALE);
+        icon_box_height = round (window_height * ICON_SCALE);
+        label_box_width = round (window_width);
+        label_box_height = round (window_height * 0.175);
 
-        icon_box_x0 = (window_width - icon_box_width) / 2;
-        icon_box_y0 = (window_height - icon_box_height - bright_box_height) / 2;
-        bright_box_x0 = round (icon_box_x0);
-        bright_box_y0 = round (icon_box_height + icon_box_y0);
+        icon_box_x0 = round ((window_width - icon_box_width) / 2);
+        icon_box_y0 = round ((window_height - icon_box_height) / 2);
+        label_box_y0 = round (window_height - label_box_height / 2);
 
 #if 0
         g_message ("icon box: w=%f h=%f _x0=%f _y0=%f",
@@ -602,11 +699,10 @@ draw_action_custom (MsdMediaKeysWindow *window,
                    icon_box_height,
                    icon_box_x0,
                    icon_box_y0);
-        g_message ("brightness box: w=%f h=%f _x0=%f _y0=%f",
-                   bright_box_width,
-                   bright_box_height,
-                   bright_box_x0,
-                   bright_box_y0);
+        g_message ("label box: w=%f h=%f _x0=0.0 _y0=%f",
+                   label_box_width,
+                   label_box_height,
+                   label_box_y0);
 #endif
 
         res = render_custom (window,
@@ -620,15 +716,12 @@ draw_action_custom (MsdMediaKeysWindow *window,
                             icon_box_width, icon_box_height);
         }
 
-        if (window->priv->show_level != FALSE) {
-                /* draw volume meter */
-                draw_volume_boxes (window,
-                                   cr,
-                                   (double)window->priv->volume_level / 100.0,
-                                   bright_box_x0,
-                                   bright_box_y0,
-                                   bright_box_width,
-                                   bright_box_height);
+        if (window->priv->description != NULL) {
+                /* draw description label meter */
+                draw_description_label (window,
+                                        cr,
+                                        label_box_y0,
+                                        label_box_width);
         }
 }
 
@@ -678,6 +771,7 @@ msd_media_keys_window_init (MsdMediaKeysWindow *window)
 
                 window->priv->image = GTK_IMAGE (gtk_builder_get_object (builder, "acme_image"));
                 window->priv->progress = GTK_WIDGET (gtk_builder_get_object (builder, "acme_volume_progressbar"));
+                window->priv->label = GTK_WIDGET (gtk_builder_get_object (builder, "acme_label"));
                 box = GTK_WIDGET (gtk_builder_get_object (builder, "acme_box"));
 
                 if (box != NULL) {
